@@ -178,10 +178,11 @@ static gboolean
 meta_display_handle_event (MetaDisplay        *display,
                            const ClutterEvent *event)
 {
+  MetaBackend *backend = meta_get_backend ();
   MetaWindow *window;
   gboolean bypass_clutter = FALSE;
   G_GNUC_UNUSED gboolean bypass_wayland = FALSE;
-  MetaGestureTracker *tracker;
+  MetaGestureTracker *gesture_tracker;
   ClutterEventSequence *sequence;
   ClutterInputDevice *source;
 
@@ -221,12 +222,8 @@ meta_display_handle_event (MetaDisplay        *display,
       (event->type == CLUTTER_PAD_BUTTON_PRESS ||
        event->type == CLUTTER_PAD_BUTTON_RELEASE))
     {
-      MetaBackend *backend = meta_get_backend ();
-
       if (meta_input_settings_handle_pad_button (meta_backend_get_input_settings (backend),
-                                                 clutter_event_get_source_device (event),
-                                                 event->type == CLUTTER_PAD_BUTTON_PRESS,
-                                                 event->pad_button.button))
+                                                 &event->pad_button))
         {
           bypass_wayland = bypass_clutter = TRUE;
           goto out;
@@ -237,7 +234,7 @@ meta_display_handle_event (MetaDisplay        *display,
 
   if (source)
     {
-      meta_backend_update_last_device (meta_get_backend (),
+      meta_backend_update_last_device (backend,
                                        clutter_input_device_get_device_id (source));
     }
 
@@ -254,8 +251,12 @@ meta_display_handle_event (MetaDisplay        *display,
         }
       else
         {
-          MetaCursorTracker *tracker = meta_cursor_tracker_get_for_screen (NULL);
-          meta_cursor_tracker_update_position (tracker, event->motion.x, event->motion.y);
+          MetaCursorTracker *cursor_tracker =
+            meta_backend_get_cursor_tracker (backend);
+
+          meta_cursor_tracker_update_position (cursor_tracker,
+                                               event->motion.x,
+                                               event->motion.y);
         }
 
       display->monitor_cache_invalidated = TRUE;
@@ -290,9 +291,9 @@ meta_display_handle_event (MetaDisplay        *display,
         }
     }
 
-  tracker = meta_display_get_gesture_tracker (display);
+  gesture_tracker = meta_display_get_gesture_tracker (display);
 
-  if (meta_gesture_tracker_handle_event (tracker, event))
+  if (meta_gesture_tracker_handle_event (gesture_tracker, event))
     {
       bypass_wayland = bypass_clutter = TRUE;
       goto out;
@@ -372,7 +373,6 @@ meta_display_handle_event (MetaDisplay        *display,
            * have the synchronous grab. */
           if (event->type == CLUTTER_BUTTON_PRESS)
             {
-              MetaBackend *backend = meta_get_backend ();
               if (META_IS_BACKEND_X11 (backend))
                 {
                   Display *xdisplay = meta_backend_x11_get_xdisplay (META_BACKEND_X11 (backend));
