@@ -327,6 +327,14 @@ meta_monitor_manager_test_tiled_monitor_removed (MetaMonitorManager *manager,
   manager_test->tiled_monitor_count--;
 }
 
+static gboolean
+meta_monitor_manager_test_is_transform_handled (MetaMonitorManager  *manager,
+                                                MetaCrtc            *crtc,
+                                                MetaMonitorTransform transform)
+{
+  return TRUE;
+}
+
 static void
 meta_monitor_manager_test_dispose (GObject *object)
 {
@@ -358,4 +366,5 @@ meta_monitor_manager_test_class_init (MetaMonitorManagerTestClass *klass)
   manager_class->apply_configuration = meta_monitor_manager_test_apply_configuration;
   manager_class->tiled_monitor_added = meta_monitor_manager_test_tiled_monitor_added;
   manager_class->tiled_monitor_removed = meta_monitor_manager_test_tiled_monitor_removed;
+  manager_class->is_transform_handled = meta_monitor_manager_test_is_transform_handled;
 }
