@@ -27,6 +27,9 @@
 #include "meta-wayland-buffer.h"
 #include "meta-xwayland.h"
 #include "screen-private.h"
+#include "meta-wayland-private.h"
+#include "backends/meta-backend-private.h"
+#include "backends/meta-logical-monitor.h"
 
 typedef struct _MetaWaylandSurfaceRoleCursorPrivate MetaWaylandSurfaceRoleCursorPrivate;
 
@@ -37,6 +40,8 @@ struct _MetaWaylandSurfaceRoleCursorPrivate
   MetaCursorSprite *cursor_sprite;
   MetaCursorRenderer *cursor_renderer;
   MetaWaylandBuffer *buffer;
+  struct wl_list frame_callbacks;
+  gulong cursor_painted_handler_id;
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (MetaWaylandSurfaceRoleCursor,
@@ -93,16 +98,23 @@ cursor_sprite_prepare_at (MetaCursorSprite             *cursor_sprite,
 {
   MetaWaylandSurfaceRole *role = META_WAYLAND_SURFACE_ROLE (cursor_role);
   MetaWaylandSurface *surface = meta_wayland_surface_role_get_surface (role);
-  MetaDisplay *display = meta_get_display ();
-  MetaScreen *screen = display->screen;
-  const MetaMonitorInfo *monitor;
 
   if (!meta_xwayland_is_xwayland_surface (surface))
     {
-      monitor = meta_screen_get_monitor_for_point (screen, x, y);
-      if (monitor)
-        meta_cursor_sprite_set_texture_scale (cursor_sprite,
-                                              (float) monitor->scale / surface->scale);
+      MetaBackend *backend = meta_get_backend ();
+      MetaMonitorManager *monitor_manager =
+        meta_backend_get_monitor_manager (backend);
+      const MetaLogicalMonitor *logical_monitor;
+
+      logical_monitor =
+        meta_monitor_manager_get_logical_monitor_at (monitor_manager, x, y);
+      if (logical_monitor)
+        {
+          float texture_scale;
+
+          texture_scale = (float) logical_monitor->scale / surface->scale;
+          meta_cursor_sprite_set_texture_scale (cursor_sprite, texture_scale);
+        }
     }
   meta_wayland_surface_update_outputs (surface);
 }
@@ -110,21 +122,16 @@ cursor_sprite_prepare_at (MetaCursorSprite             *cursor_sprite,
 static void
 cursor_surface_role_assigned (MetaWaylandSurfaceRole *surface_role)
 {
+  MetaWaylandSurface *surface =
+    meta_wayland_surface_role_get_surface (surface_role);
   MetaWaylandSurfaceRoleCursor *cursor_role =
     META_WAYLAND_SURFACE_ROLE_CURSOR (surface_role);
   MetaWaylandSurfaceRoleCursorPrivate *priv =
     meta_wayland_surface_role_cursor_get_instance_private (cursor_role);
-  MetaWaylandSurface *surface =
-    meta_wayland_surface_role_get_surface (surface_role);
-  MetaWaylandBuffer *buffer = meta_wayland_surface_get_buffer (surface);
 
-  if (buffer)
-    {
-      g_set_object (&priv->buffer, buffer);
-      meta_wayland_surface_ref_buffer_use_count (surface);
-    }
-
-  meta_wayland_surface_queue_pending_frame_callbacks (surface);
+  wl_list_insert_list (&priv->frame_callbacks,
+                       &surface->pending_frame_callback_list);
+  wl_list_init (&surface->pending_frame_callback_list);
 }
 
 static void
@@ -164,15 +171,17 @@ cursor_surface_role_commit (MetaWaylandSurfaceRole  *surface_role,
         meta_wayland_surface_ref_buffer_use_count (surface);
     }
 
-  meta_wayland_surface_queue_pending_state_frame_callbacks (surface, pending);
+  wl_list_insert_list (&priv->frame_callbacks,
+                       &pending->frame_callback_list);
+  wl_list_init (&pending->frame_callback_list);
 
   if (pending->newly_attached)
     update_cursor_sprite_texture (META_WAYLAND_SURFACE_ROLE_CURSOR (surface_role));
 }
 
 static gboolean
-cursor_surface_role_is_on_output (MetaWaylandSurfaceRole *role,
-                                  MetaMonitorInfo        *monitor)
+cursor_surface_role_is_on_logical_monitor (MetaWaylandSurfaceRole *role,
+                                           MetaLogicalMonitor     *logical_monitor)
 {
   MetaWaylandSurface *surface =
     meta_wayland_surface_role_get_surface (role);
@@ -184,7 +193,7 @@ cursor_surface_role_is_on_output (MetaWaylandSurfaceRole *role,
 
   rect = meta_cursor_renderer_calculate_rect (priv->cursor_renderer,
                                               priv->cursor_sprite);
-  return meta_rectangle_overlap (&rect, &monitor->rect);
+  return meta_rectangle_overlap (&rect, &logical_monitor->rect);
 }
 
 static void
@@ -196,6 +205,10 @@ cursor_surface_role_dispose (GObject *object)
     meta_wayland_surface_role_cursor_get_instance_private (cursor_role);
   MetaWaylandSurface *surface =
     meta_wayland_surface_role_get_surface (META_WAYLAND_SURFACE_ROLE (object));
+  MetaWaylandFrameCallback *cb, *next;
+
+  wl_list_for_each_safe (cb, next, &priv->frame_callbacks, link)
+    wl_resource_destroy (cb->resource);
 
   g_signal_handlers_disconnect_by_func (priv->cursor_sprite,
                                         cursor_sprite_prepare_at, cursor_role);
@@ -212,6 +225,30 @@ cursor_surface_role_dispose (GObject *object)
   G_OBJECT_CLASS (meta_wayland_surface_role_cursor_parent_class)->dispose (object);
 }
 
+static void
+cursor_surface_role_constructed (GObject *object)
+{
+  MetaWaylandSurfaceRoleCursor *cursor_role =
+    META_WAYLAND_SURFACE_ROLE_CURSOR (object);
+  MetaWaylandSurfaceRoleCursorPrivate *priv =
+    meta_wayland_surface_role_cursor_get_instance_private (cursor_role);
+  MetaWaylandSurfaceRole *surface_role =
+    META_WAYLAND_SURFACE_ROLE (cursor_role);
+  MetaWaylandSurface *surface =
+    meta_wayland_surface_role_get_surface (surface_role);
+  MetaWaylandBuffer *buffer;
+
+  buffer = meta_wayland_surface_get_buffer (surface);
+
+  g_warn_if_fail (!buffer || buffer->resource);
+
+  if (buffer && buffer->resource)
+    {
+      g_set_object (&priv->buffer, buffer);
+      meta_wayland_surface_ref_buffer_use_count (surface);
+    }
+}
+
 static void
 meta_wayland_surface_role_cursor_init (MetaWaylandSurfaceRoleCursor *role)
 {
@@ -224,6 +261,7 @@ meta_wayland_surface_role_cursor_init (MetaWaylandSurfaceRoleCursor *role)
                            G_CALLBACK (cursor_sprite_prepare_at),
                            role,
                            0);
+  wl_list_init (&priv->frame_callbacks);
 }
 
 static void
@@ -236,8 +274,9 @@ meta_wayland_surface_role_cursor_class_init (MetaWaylandSurfaceRoleCursorClass *
   surface_role_class->assigned = cursor_surface_role_assigned;
   surface_role_class->pre_commit = cursor_surface_role_pre_commit;
   surface_role_class->commit = cursor_surface_role_commit;
-  surface_role_class->is_on_output = cursor_surface_role_is_on_output;
+  surface_role_class->is_on_logical_monitor = cursor_surface_role_is_on_logical_monitor;
 
+  object_class->constructed = cursor_surface_role_constructed;
   object_class->dispose = cursor_surface_role_dispose;
 }
 
@@ -281,6 +320,28 @@ meta_wayland_surface_role_cursor_get_hotspot (MetaWaylandSurfaceRoleCursor *curs
     *hotspot_y = priv->hot_y;
 }
 
+static void
+on_cursor_painted (MetaCursorRenderer           *renderer,
+                   MetaCursorSprite             *displayed_sprite,
+                   MetaWaylandSurfaceRoleCursor *cursor_role)
+{
+  MetaWaylandSurfaceRoleCursorPrivate *priv =
+    meta_wayland_surface_role_cursor_get_instance_private (cursor_role);
+  guint32 time = (guint32) (g_get_monotonic_time () / 1000);
+
+  if (displayed_sprite != priv->cursor_sprite)
+    return;
+
+  while (!wl_list_empty (&priv->frame_callbacks))
+    {
+      MetaWaylandFrameCallback *callback =
+        wl_container_of (priv->frame_callbacks.next, callback, link);
+
+      wl_callback_send_done (callback->resource, time);
+      wl_resource_destroy (callback->resource);
+    }
+}
+
 void
 meta_wayland_surface_role_cursor_set_renderer (MetaWaylandSurfaceRoleCursor *cursor_role,
                                                MetaCursorRenderer           *renderer)
@@ -291,10 +352,20 @@ meta_wayland_surface_role_cursor_set_renderer (MetaWaylandSurfaceRoleCursor *cur
   if (priv->cursor_renderer == renderer)
     return;
 
-  if (renderer)
-    g_object_ref (renderer);
   if (priv->cursor_renderer)
-    g_object_unref (priv->cursor_renderer);
+    {
+      g_signal_handler_disconnect (priv->cursor_renderer,
+                                   priv->cursor_painted_handler_id);
+      priv->cursor_painted_handler_id = 0;
+      g_object_unref (priv->cursor_renderer);
+    }
+  if (renderer)
+    {
+      priv->cursor_painted_handler_id =
+        g_signal_connect_object (renderer, "cursor-painted",
+                                 G_CALLBACK (on_cursor_painted), cursor_role, 0);
+      g_object_ref (renderer);
+    }
 
   priv->cursor_renderer = renderer;
   update_cursor_sprite_texture (cursor_role);
